

**************************************************************************************************
**** PROGRAM NAME:  ParcelAlloc.sas                                                             **
**** AUTHOR:        Sonya Sterba and Robert MacCallum (2010)                                    **
**** LAST MODIFIED: February, 2010                                                               **
**** FOR FULL DOCUMENTATION AND COMMAND LINES INPUT:   http://www.unc.edu/~ssterba/parcel.htm   **
**************************************************************************************************;

/*The next two lines turn off SAS's log file and notes, among other things;
  comment them out if you want to see this information*/
ods select none; ods noresults; 
options NOMLOGIC NOMPRINT NOMRECALL NOSYMBOLGEN nosource nonotes;

%MACRO PARCELALLOC; 

LIBNAME PARCEL "&directory.";

data simdat; set &inputitemdataset.; run;

%DO R=1 %TO &AREP.;
%PUT "Macro is processing parcel-allocation &r.";

	%macro parcelgen;
	%do f=1 %to &factors.;
		data start;
		/*randomly sort items within factor*/
		do i=1 to &&numitemsfac&f;
  			y = ranuni(-1); output; 
		end; run;
		PROC RANK data=start  out=ranks2 descending; VAR y; RUN;
		data ranks3; set ranks2; item="item";  
		yvars=cats(item,"f&f.y");
		id=cats("f&f.y",i); run;
		proc transpose data=ranks3 out=transposeranks3;
  		id id;
  		var yvars; run;


		data ranks22; set ranks2; id=cats("f&f.y",i); run;
		proc transpose data=ranks22 out=transposeranks22;
		id id; var y; run;
		data transposeranks22f&f.; set transposeranks22; drop _NAME_ _LABEL_; run;

        /*This macro extracts the parcel sizes from user input vector*/
 		%macro rename; 
  		%do t=1 %to &N.;
			%local i var; 
			%let i=0; 
			%do %while(%scan(&&varlist&f,&i.+1,%str( )) ne %str( )); /* The %DO %WHILE loops through the elements in the array, one word at a time. */ 
				%let i = %eval(&i.+1);    /*Each time a word is found, the index variable (&i) is incremented by one.*/ 
				%let var = %scan(&&varlist&f,&i.,%str( )); /*A temporary macro variable is used to hold the current word in the list.*/
 				%if &i.=1 %then %do;
					data new&t.; 
					f&f.parcel&i.=&var.;  run; 
  				%end;
 				%else %do;
					data new&t.; set new&t.;
					f&f.parcel&i.=&var.;  run;
  				%end;
			%end; 
 			%IF &t. = 1 %THEN %DO;
 				data newf&f.; merge transposeranks22f&f. new&t.; run;
			%END;
  			%ELSE %DO;
 				data temp; merge transposeranks22f&f. new&t.;  run;
 				data newf&f.; set newf&f. temp; run;
			%END;
		%end;
		%mend rename; 
  		%rename 
	%end; *factor loop;

	
	%if &r.=1 %then %do;
		%macro fred;
		data allo1; set transposeranks22f1; run; 
		%if &factors.>1 %then %do;
		%do f=2 %to &factors.;
		%let e=%eval(&f.-1);
		data allo&f.; merge transposeranks22f&f. allo&e.; run; %end;
		%end;
		data allocations; set allo&factors.;  replication=&r. ;run;
		%mend;
		%fred;
	%end;
	%else %do;
	   %macro fred;
		data allo1; set transposeranks22f1; run; 
		%if &factors.>1 %then %do;
		%do f=2 %to &factors.;
		%let e=%eval(&f.-1);
		data allo&f.; merge transposeranks22f&f. allo&e.; run; %end;
		%end;
		data alloctemp; set allo&factors.;  replication=&r.; run;
		%mend;
		%fred;
		data allocations; set allocations alloctemp; run;
	%end;


  	%macro consolidate;
  	%do f=1 %to &factors.;
 		newf&f.
  	%end;
  	%mend consolidate;

  	data tootle; merge simdat %consolidate ; run;
	%macro simo;
	%do f=1 %to &factors.;
		data tootle; set tootle;
		array f&f.parcelvec{&&numparcelsfac&f} f&f.parcel1-f&f.parcel&&numparcelsfac&f; 
		array f&f.itemlist{&&numitemsfac&f} f&f.y1-f&f.y&&numitemsfac&f;
		array f&f.item {&&numitemsfac&f};
		n=0;
    	%do j=1 %to &&numparcelsfac&f;   
			m=n+1;                    
			n=m+f&f.parcelvec{&j.}-1;   
			f&f.meanparcel&j.=0; 

			counter=0;                   
			do k=m to n;                 
				e=f&f.itemlist{k};
				if f&f.item{e} NE &missing. then do;
					counter=counter+1;  
					f&f.meanparcel&j.=f&f.meanparcel&j.+f&f.item{e};   
				end;
			end;
			if counter NE 0 then do;
				f&f.parc&j.=f&f.meanparcel&j./counter;   
				end;
			else do;
				f&f.parc&j.= &missing.;             
			end;
			drop f&f.meanparcel&j.;                           
		%end;

	%end;
	run;
	%mend simo;
	%simo;
	

 	%macro preen;
  	%do f=1 %to &factors.;
  		%do g=1 %to &&numitemsfac&f;
  			f&f.item&g.
  			f&f.y&g.
  			%do h=1 %to &&numparcelsfac&f;
  				f&f.parcel&h.
  			%end;
  		%end;
   %end;
   %mend preen;
   data tootle2; set tootle;
   drop %preen n m k e counter; run;

   %macro listed;
   %do f=1 %to &factors.;
  		%do g=1 %to &&numparcelsfac&f;
  			f&f.parc&g.
  		%end;
   %end;
   %mend listed;

   DATA _NULL_; SET tootle2;
   FILE "&directory.parcelruns&r..dat";
   PUT (_ALL_) (F16.6,''X);RUN;


	data parcelrunsreplist&r.;
	length list $ 30;
	list="parcelruns&r..dat"; run;

	%if &r.=1 %then %do;
		data basereplist; set parcelrunsreplist1; run;
	%end;
	%else  %do;
		proc append base=basereplist data=parcelrunsreplist&r.; run;
	%end;

%mend parcelgen;
%PARCELGEN;
%end; *arep loop;

data parcelrunsreplist; set basereplist; run;

 DATA _NULL_; SET parcelrunsreplist;
   FILE "&directory.parcelrunsreplist.dat";
   PUT (list) (F19.6,''X);RUN;

%macro namelist(tootle2);
proc contents data=&tootle2. out=cont noprint; run;
proc sort data=cont; by VARNUM;  run;
proc sql noprint; select distinct NAME into: varname1-:varname999 from cont ORDER by VARNUM; quit;
%put The variables in each parcelruns data file appear in this order:; 
%do i=1 %to &sqlobs; 
%put &i &&varname&i; %end; %mend namelist;
%namelist(tootle2);

data parcel.allocations; set allocations; run;

%MEND PARCELALLOC;

%PARCELALLOC;


